#!/usr/bin/perl -w
#
use strict;
#
####################################################################
#
# Name: createKML.pl - a script to generate a KML file based on a
#       detection file
#
# Copyright (C) 2012  Product Team, IRIS Data Management Center 
#
#    This is a free software; you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as
#    published by the Free Software Foundation; either version 3 of the
#    License, or (at your option) any later version.
#    
#    This software is distributed in the hope that it will be useful, but
#    WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License (GNU-LGPL) for more details.  The
#    GNU-LGPL and further information can be found here:
#    http://www.gnu.org/
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Description:
#
#    Script reads a detection file and creates a KML output file.
#    The script is currently designed to accept customized PSD detections.
#    To accept other detection formats, simply modify the input statement.
#
# Input:
#
#    Script expects the station and the detection files in the same directory as the script
#
#       1. from http://www.iris.edu/dms/products/infrasound/detector/repository/
#         1.1 download the weekly detection file (customized power spectra detections) of interest
#         1.2 download the station file of interest (stationList.txt)
#
#       2. edit the detection file and remove the header information (the first line must start with a detection record)
#
#       3. If desired, use the "grep" command on the detection file to extract only
#          the data days of interest
#
#       4. the yellow-dot.png file, included in this bundle, must also reside in the same directory
#   
# Example:
#
#     createKML.pl <detection file name> <kml file name>
#
#     with the station file "stationList.txt" and the detection files in the same directory as the
#     script, run the script:
#
#     createKML.pl IRIS_TAID_2012-10-16.txt IRIS_TAID_2012-10-16.kml
#    
# History:
#
#    2012-10-30 (MB): created
#
#
#
# 

###############################################################################
# trim blanks
################################################################################
sub Trim
   {
   my $line = $_[0];
   chomp($line);
   $line =~ s/^\s+//;
   $line =~ s/\s+$//;
   return $line;
   }

###############################################################################
# MAIN
################################################################################

#
# get user parameters
#
my $stationFileName = "./stationList.txt";

unless ($#ARGV != 2)
   {
   print STDERR "[ERR] expected detection and KML file names\n";
   exit;
   }
my $detectionFileName = $ARGV[0];
chomp $detectionFileName;
my $kmlFileName       = $ARGV[1];
chomp $kmlFileName;

#
# read and store station coordinates
#
unless (-e $stationFileName) 
   {
   print STDERR "[ERR] could not find ".$stationFileName."\n";
   exit;
   }
open STATIONFILE , "<  $stationFileName";

my $header = <STATIONFILE>;
$header = <STATIONFILE>;

my %latitude  = ();
my %longitude = ();
while (<STATIONFILE>)
   {
   $_ = Trim($_);
   my ($sta,$lat,$lon) = split('\|',$_);
   $sta = Trim($sta);
   $lat = Trim($lat);
   $latitude{$sta}  = $lat;
   $lon = Trim($lon);
   $longitude{$sta} = $lon;
   }
close STATIONFILE;

#
# print header of KML
#
open KMLFILE       , ">  $kmlFileName";
print KMLFILE "<?xml version=\"1.0\" encoding=\"UTF-8\"?> \n";
print KMLFILE "<kml xmlns=\"http://earth.google.com/kml/2.0\"> <Document>";
for (my $i=1;$i<=20;$i++)
   {
   printf KMLFILE "<Style id=\"style_%0.1f\">\n",$i/10.0;
   print KMLFILE "<IconStyle> <Icon> <href>./yellow-dot.png</href> \n";
   print KMLFILE "</Icon>\n";
   printf KMLFILE "<scale>%0.1f</scale>\n",$i/10.0;
   print KMLFILE "</IconStyle>\n";
   print KMLFILE "</Style>\n"; 
   }

#
# read and parse detection file
#
open DETECTIONFILE , "<  $detectionFileName";
my $count = 0;
while (<DETECTIONFILE>)
   {
   $count++;
   $_ = Trim($_);
   my ($sta,$start,$back,$peak,$t) = split('\|',$_);
   $sta     = Trim($sta);
   $start   = Trim($start);
   $start   =~ s/ /T/;
   $back    = Trim($back);
   $t       = Trim($t);
   $t       =~ s/ /T/;
   my $diff =$peak -  $back;
   next if ($diff <= 0);
   my $scale = log($diff)/log(10);
   $scale = 2 if ($scale > 2.0);
   my $iconScale = sprintf("%0.1f",$scale);
   if($longitude{$sta} && $latitude{$sta})   
      {
      print KMLFILE "<Placemark>\n";
      print KMLFILE "<TimeStamp> \n";
      print KMLFILE "<when>".$t."Z</when>\n";
      print KMLFILE "</TimeStamp>\n";
      print KMLFILE "<styleUrl> #style_".$iconScale."</styleUrl>\n"; 
      print KMLFILE "<Point>\n";
      print KMLFILE "<coordinates>$longitude{$sta},$latitude{$sta}</coordinates>\n";
      print KMLFILE "</Point>\n";
      print KMLFILE "</Placemark>\n";
      }
   }
close DETECTIONFILE;

print KMLFILE "</Document></kml>\n";
close KMLFILE;

